@@ -1,30 +1,39 @@
 """
-VASP calculation.
+This module contains the class that prepares a specific VASP calculation.
+"""
 
------------------
-The calculation class that prepares a specific VASP calculation.
-"""
-#encoding: utf-8
+from __future__ import annotations
+
+# encoding: utf-8
 # pylint: disable=abstract-method
 # explanation: pylint wrongly complains about (aiida) Node not implementing query
 import os
+from typing import TYPE_CHECKING, Any
 
+from aiida import orm
 from aiida.common.exceptions import InputValidationError, ValidationError
+from aiida.engine import ProcessBuilder
+from aiida.orm.nodes.data.base import to_aiida_type
 
 from aiida_vasp.calcs.base import VaspCalcBase
+from aiida_vasp.data.chargedensity import ChargedensityData
+from aiida_vasp.data.potcar import PotcarData
+from aiida_vasp.data.wavefun import WavefunData
 from aiida_vasp.parsers.content_parsers.incar import IncarParser
 from aiida_vasp.parsers.content_parsers.kpoints import KpointsParser
 from aiida_vasp.parsers.content_parsers.poscar import PoscarParser
 from aiida_vasp.parsers.content_parsers.potcar import MultiPotcarIo
-from aiida_vasp.utils.aiida_utils import get_data_class, get_data_node
+
+if TYPE_CHECKING:
+    from aiida.common import CalcInfo
+    from aiida.common.folders import Folder
 
 
 class VaspCalculation(VaspCalcBase):
     """
     General-purpose VASP calculation.
 
-    ---------------------------------
-    By default retrieves only the 'OUTCAR', 'vasprun.xml', 'EIGENVAL', 'DOSCAR'
+    By default retrieves only the 'OUTCAR', 'vasprun.xml', 'CONTCAR'
     and Wannier90 input / output objects. These objects are deleted after parsing.
     Additional retrieve objects can be specified via the
     ``settings['ADDITIONAL_RETRIEVE_TEMPORARY_LIST']`` input. In addition, if you want to keep
@@ -62,29 +71,31 @@ class VaspCalculation(VaspCalcBase):
     """
 
     _VASP_OUTPUT = 'vasp_output'
-    _ALWAYS_RETRIEVE_LIST = ['CONTCAR', 'OUTCAR', 'vasprun.xml', 'EIGENVAL', 'DOSCAR', 'wannier90*', _VASP_OUTPUT]
+    _ALWAYS_RETRIEVE_LIST = ['CONTCAR', 'OUTCAR', 'vasprun.xml', 'wannier90*', _VASP_OUTPUT]
     _query_type_string = 'vasp.vasp'
     _plugin_type_string = 'vasp.vasp'
 
     @classmethod
-    def define(cls, spec):
+    def define(cls, spec: Any) -> None:
         super(VaspCalculation, cls).define(spec)
         # Define the inputs.
         # options is passed automatically.
         spec.input(
             'parameters',
-            valid_type=get_data_class('core.dict'),
+            valid_type=orm.Dict,
+            serializer=to_aiida_type,
             help='The VASP input parameters (INCAR).',
         )
         spec.input(
             'dynamics',
-            valid_type=get_data_class('core.dict'),
+            valid_type=orm.Dict,
+            serializer=to_aiida_type,
             help='The VASP parameters related to ionic dynamics, e.g. flags to set the selective dynamics',
             required=False,
         )
         spec.input(
             'structure',
-            valid_type=(get_data_class('core.structure'), get_data_class('core.cif')),
+            valid_type=(orm.StructureData, orm.CifData),
             help='The input structure (POSCAR).',
         )
 
@@ -92,30 +103,31 @@ class VaspCalculation(VaspCalcBase):
         # until execution.
         spec.input_namespace(
             'potential',
-            valid_type=get_data_class('vasp.potcar'),
+            valid_type=PotcarData,
             help='The potentials (POTCAR).',
             dynamic=True,
         )
         spec.input(
             'kpoints',
-            valid_type=get_data_class('core.array.kpoints'),
+            valid_type=orm.KpointsData,
             help='The kpoints to use (KPOINTS).',
         )
         spec.input(
             'charge_density',
-            valid_type=get_data_class('vasp.chargedensity'),
+            valid_type=ChargedensityData,
             required=False,
             help='The charge density. (CHGCAR)',
         )
         spec.input(
             'wavefunctions',
-            valid_type=get_data_class('vasp.wavefun'),
+            valid_type=WavefunData,
             required=False,
             help='The wave function coefficients. (WAVECAR)',
         )
         spec.input(
             'settings',
-            valid_type=get_data_class('core.dict'),
+            valid_type=orm.Dict,
+            serializer=to_aiida_type,
             required=False,
             help='Additional parameters not related to VASP itself.',
         )
@@ -123,123 +135,93 @@ class VaspCalculation(VaspCalcBase):
             'metadata.options.parser_name',
             default='vasp.vasp',
         )
+        spec.input(
+            'vdw_kernel',
+            valid_type=orm.SinglefileData,
+            required=False,
+            help='The vdw_kerenl.bindat file to be used for vdw calculations.',
+        )
 
         # Define outputs.
         # remote_folder and retrieved are passed automatically
         spec.output(
             'misc',
-            valid_type=get_data_class('core.dict'),
+            valid_type=orm.Dict,
             help='The output parameters containing smaller quantities that do not depend on system size.',
         )
+        # Mark misc as the default output node
+        spec.default_output_node = 'misc'
         spec.output(
             'structure',
-            valid_type=get_data_class('core.structure'),
+            valid_type=orm.StructureData,
             required=False,
             help='The output structure.',
         )
         spec.output(
             'kpoints',
-            valid_type=get_data_class('core.array.kpoints'),
+            valid_type=orm.KpointsData,
             required=False,
             help='The output k-points.',
         )
         spec.output(
             'trajectory',
-            valid_type=get_data_class('core.array.trajectory'),
+            valid_type=orm.TrajectoryData,
+            required=False,
+            help='The output trajectory data.',
+        )
+        spec.output(
+            'arrays',
+            valid_type=orm.ArrayData,
             required=False,
             help='The output trajectory data.',
         )
         spec.output(
             'chgcar',
-            valid_type=get_data_class('vasp.chargedensity'),
+            valid_type=ChargedensityData,
             required=False,
             help='The output charge density CHGCAR file.',
         )
         spec.output(
             'wavecar',
-            valid_type=get_data_class('vasp.wavefun'),
+            valid_type=WavefunData,
             required=False,
             help='The output plane wave coefficients file.',
         )
         spec.output(
             'bands',
-            valid_type=get_data_class('core.array.bands'),
+            valid_type=orm.BandsData,
             required=False,
             help='The output band structure.',
         )
-        spec.output(
-            'forces',
-            valid_type=get_data_class('core.array'),
-            required=False,
-            help='The output forces.',
-        )
-        spec.output(
-            'stress',
-            valid_type=get_data_class('core.array'),
-            required=False,
-            help='The output stress.',
-        )
         spec.output(
             'dos',
-            valid_type=get_data_class('core.array'),
+            valid_type=orm.ArrayData,
             required=False,
             help='The output dos.',
         )
         spec.output(
             'energies',
-            valid_type=get_data_class('core.array'),
+            valid_type=orm.ArrayData,
             required=False,
-            help='The output total energies.',
+            help='Energies of the calculation at each ionic/electronic step.',
         )
         spec.output(
             'projectors',
-            valid_type=get_data_class('core.array'),
+            valid_type=orm.ArrayData,
             required=False,
-            help='The output projectors of decomposition.',
+            help='The projectors for the calculation.',
         )
         spec.output(
-            'dielectrics',
-            valid_type=get_data_class('core.array'),
-            required=False,
-            help='The output dielectric functions.',
+            'parameters', valid_type=orm.Dict, required=False, help='All input parameters including the default values.'
         )
-        spec.output(
-            'born_charges',
-            valid_type=get_data_class('core.array'),
-            required=False,
-            help='The output Born effective charges.',
-        )
-        spec.output(
-            'hessian',
-            valid_type=get_data_class('core.array'),
-            required=False,
-            help='The output Hessian matrix.',
-        )
-        spec.output(
-            'dynmat',
-            valid_type=get_data_class('core.array'),
-            required=False,
-            help='The output dynamical matrix.',
-        )
-        spec.output(
-            'charge_density',
-            valid_type=get_data_class('core.array'),
-            required=False,
-            help='The output charge density.',
-        )
-        spec.output(
-            'magnetization_density',
-            valid_type=get_data_class('core.array'),
-            required=False,
-            help='The output magnetization density.',
-        )
-        spec.output(
-            'site_magnetization',
-            valid_type=get_data_class('core.dict'),
-            required=False,
-            help='The output of the site magnetization',
-        )
-        spec.output_namespace('custom_outputs', required=False, dynamic=True)
+        # Standalone array quantities
+        for name in ['hessian', 'dynmat', 'born_charges', 'dielectrics']:
+            spec.output(
+                name,
+                valid_type=orm.ArrayData,
+                required=False,
+                help='The output {name} data.',
+            )
         spec.exit_code(0, 'NO_ERROR', message='the sun is shining')
         spec.exit_code(
             350,
@@ -291,26 +273,14 @@ class VaspCalculation(VaspCalcBase):
         spec.exit_code(
             704,
             'ERROR_DIAGNOSIS_OUTPUTS_MISSING',
-            message=
-            'Outputs for diagnosis are missing, please make sure `run_status` and `notifications` quantities are requested for parsing.',
+            message='Outputs for diagnosis are missing, please make sure `run_status` and `notifications`'
+            ' quantities are requested for parsing.',
         )
 
-        spec.exit_code(
-            1001,
-            'ERROR_PARSING_OBJECT_FAILED',
-            message='parsing an object has failed.',
-        )
         spec.exit_code(
             1002,
             'ERROR_NOT_ABLE_TO_PARSE_QUANTITY',
-            message='the parser is not able to parse the {quantity} quantity',
-        )
-        spec.exit_code(
-            1003,
-            'ERROR_RECOVERY_PARSING_OF_XML_FAILED',
-            message=
-            'the vasprun.xml was truncated and recovery parsing failed to parse at least one of the requested quantities: {quantities}, '
-            'very likely the VASP calculation did not run properly',
+            message='the parser is not able to parse the {quantity} quantity which is marked as required',
         )
         spec.exit_code(
             1004,
@@ -319,7 +289,7 @@ class VaspCalculation(VaspCalcBase):
         )
         spec.exit_code(1005, 'ERROR_OVERFLOW_IN_XML', message='Overflow detected in XML while parsing.')
 
-    def prepare_for_submission(self, folder):
+    def prepare_for_submission(self, folder: Folder) -> CalcInfo:
         """
         Add all objects to the list of objects to be retrieved.
 
@@ -343,15 +313,16 @@ class VaspCalculation(VaspCalcBase):
         except AttributeError:
             additional_retrieve_list = []
         try:
-            additional_retrieve_temp_list =\
-                self.inputs.settings.base.attributes.get('ADDITIONAL_RETRIEVE_TEMPORARY_LIST', default=[])  # pylint: disable=invalid-name
+            additional_retrieve_temp_list = self.inputs.settings.base.attributes.get(
+                'ADDITIONAL_RETRIEVE_TEMPORARY_LIST', default=[]
+            )  # pylint: disable=invalid-name
         except AttributeError:
             additional_retrieve_temp_list = []
         if store:
             calcinfo.retrieve_list = list(set(self._ALWAYS_RETRIEVE_LIST + additional_retrieve_list))
             calcinfo.retrieve_temporary_list = additional_retrieve_temp_list  # pylint: disable=invalid-name
         else:
-            calcinfo.retrieve_temporary_list = list(set(self._ALWAYS_RETRIEVE_LIST + additional_retrieve_temp_list))  # pylint: disable=invalid-name
+            calcinfo.retrieve_temporary_list = list(set(self._ALWAYS_RETRIEVE_LIST + additional_retrieve_temp_list))
             calcinfo.retrieve_list = additional_retrieve_list
         try:
             provenance_exclude_list = self.inputs.settings.base.attributes.get('PROVENANCE_EXCLUDE_LIST', default=[])
@@ -362,7 +333,7 @@ class VaspCalculation(VaspCalcBase):
 
         return calcinfo
 
-    def verify_inputs(self):
+    def verify_inputs(self) -> None:
         super().verify_inputs()
         _parameters = self.inputs.parameters.get_dict()
         _lorbit = _parameters.get('lorbit', 0)
@@ -370,12 +341,12 @@ class VaspCalculation(VaspCalcBase):
             _settings = self.inputs.settings.get_dict()
         else:
             _settings = {}
-        _site_magnetization = _settings.get('parser_settings', {}).get('add_site_magnetization', False)
+        _site_magnetization = 'site_magnetization' in _settings.get('parser_settings', {}).get('include_quantity', [])
         if _site_magnetization and _lorbit < 10:
             raise InputValidationError(f'Site magnetization requires "LORBIT>=10", value given {_lorbit}')
 
     @property
-    def _parameters(self):
+    def _parameters(self) -> dict[str, Any]:
         """Make sure all parameters are lowercase."""
         all_parameters = self.inputs.parameters.get_dict()
         try:
@@ -383,50 +354,36 @@ class VaspCalculation(VaspCalcBase):
         except KeyError:
             return {}
 
-    def _need_kp(self):
+    def _need_kp(self) -> bool:
         """
         Return wether an input kpoints node is needed or not.
 
-        :return output:
-            True if input kpoints node is needed
-            (py:method::VaspCalculation.use_kpoints),
-            False otherwise
-        needs 'parameters' input to be set
-        (py:method::VaspCalculation.use_parameters)
+        :return output: True if input kpoints node is needed False otherwise needs 'parameters' input to be set.
         """
         return not bool('kspacing' in self._parameters or 'kgamma' in self._parameters)
 
-    def _need_chgcar(self):
+    def _need_chgcar(self) -> bool:
         """
         Test wether an charge_densities input is needed or not.
 
-        :return output:
-            True if CHGCAR must be present
-            (py:method::NscfCalculation.use_charge_densities),
-            False otherwise
-        needs 'parameters' input to be set
-        (py:method::NscfCalculation.use_parameters)
+        :return output: True if CHGCAR must be present False otherwise.
+
         """
         ichrg_d = 0 if self._need_wavecar() else 2
         icharg = self._parameters.get('icharg', ichrg_d)
         return bool(icharg in [1, 11])
 
-    def _need_wavecar(self):
+    def _need_wavecar(self) -> bool:
         """
         Test wether a wavefunctions input is needed or not.
 
-        :return output:
-            True if WAVECAR must be present
-            used (py:method::NscfCalculation.use_wavefunctions),
-            False otherwise
-        needs 'parameters' input to be set
-        (py:method::NscfCalculation.use_parameters)
+        :return output: True if WAVECAR must be present used False otherwise.
         """
         istrt_d = 1 if self.inputs.get('wavefunctions') else 0
         istart = self._parameters.get('istart', istrt_d)
         return bool(istart in [1, 2, 3])
 
-    def _structure(self):
+    def _structure(self) -> orm.StructureData:
         """
         Get the input structure as AiiDa StructureData.
 
@@ -434,10 +391,10 @@ class VaspCalculation(VaspCalcBase):
         """
         structure = self.inputs.structure
         if not hasattr(structure, 'get_pymatgen'):
-            structure = get_data_node('core.structure', ase=structure.get_ase())
+            structure = orm.StructureData(ase=structure.get_ase())
         return structure
 
-    def write_additional(self, folder, calcinfo):
+    def write_additional(self, folder: Folder, calcinfo: CalcInfo) -> None:
         """Write CHGAR and WAVECAR if needed."""
         super().write_additional(folder, calcinfo)
         # a list of object names to be copied
@@ -461,7 +418,13 @@ class VaspCalculation(VaspCalcBase):
                 if 'WAVECAR' not in remote_copy_fnames:
                     raise FileNotFoundError(f'Could not find WAVECAR in {remote_folder.get_remote_path()}')
 
-    def write_incar(self, dst, validate_tags=True):  # pylint: disable=unused-argument
+        # Process the vdw_kernel input
+        if 'vdw_kernel' in self.inputs:
+            calcinfo.local_copy_list.append(
+                (self.inputs.vdw_kernel.uuid, self.inputs.vdw_kernel.filename, 'vdw_kernel.bindat')
+            )
+
+    def write_incar(self, dst: str, validate_tags: bool = True) -> None:
         """
         Write the INCAR.
 
@@ -469,6 +432,7 @@ class VaspCalculation(VaspCalcBase):
         preparation and writes to dst.
 
         :param dst: absolute path of the object to write to
+
         """
         # Check if parameters validation is turned off
         if self.inputs.get('settings'):
@@ -481,7 +445,7 @@ class VaspCalculation(VaspCalcBase):
         except SystemExit as parser_error:
             raise ValidationError('The INCAR content did not pass validation.') from parser_error
 
-    def write_poscar(self, dst):  # pylint: disable=unused-argument
+    def write_poscar(self, dst: str) -> None:
         """
         Write the POSCAR.
 
@@ -507,7 +471,7 @@ class VaspCalculation(VaspCalcBase):
         except SystemExit as parser_error:
             raise ValidationError('The POSCAR content did not pass validation.') from parser_error
 
-    def write_potcar(self, dst):
+    def write_potcar(self, dst: str) -> None:
         """
         Concatenates multiple POTCARs into one in the same order as the elements appear in POSCAR.
 
@@ -517,7 +481,7 @@ class VaspCalculation(VaspCalcBase):
         multi_potcar = MultiPotcarIo.from_structure(structure, self.inputs.potential)
         multi_potcar.write(dst)
 
-    def write_kpoints(self, dst):  # pylint: disable=unused-argument
+    def write_kpoints(self, dst: str) -> None:
         """
         Write the KPOINTS.
 
@@ -532,16 +496,16 @@ class VaspCalculation(VaspCalcBase):
         except SystemExit as parser_error:
             raise ValidationError('The KPOINTS content did not pass validation.') from parser_error
 
-    def write_chgcar(self, dst, calcinfo):  # pylint: disable=unused-argument
+    def write_chgcar(self, dst: str, calcinfo: CalcInfo) -> None:
         charge_density = self.inputs.charge_density
         calcinfo.local_copy_list.append((charge_density.uuid, charge_density.filename, dst))
 
-    def write_wavecar(self, dst, calcinfo):  # pylint: disable=unused-argument
+    def write_wavecar(self, dst: str, calcinfo: CalcInfo) -> None:
         wave_functions = self.inputs.wavefunctions
         calcinfo.local_copy_list.append((wave_functions.uuid, wave_functions.filename, dst))
 
     @classmethod
-    def immigrant(cls, code, remote_path, **kwargs):
+    def immigrant(cls, code: orm.Code, remote_path: str, **kwargs: Any) -> tuple[type, ProcessBuilder]:
         """
         Returns VaspImmigrant class and associated inputs. This method will be obsolete at v3.0
 
@@ -553,9 +517,9 @@ class VaspCalculation(VaspCalcBase):
         :param potential_mapping: dict.
         :param use_wavecar: bool. Try to read WAVECAR.
         :param use_chgcar bool. Try to read CHGCAR.
-        """
 
-        from aiida_vasp.calcs.immigrant import VaspImmigrant  # pylint: disable=import-outside-toplevel
+        """
+        from aiida_vasp.calcs.immigrant import VaspImmigrant  # noqa: PLC0415
 
         proc_cls = VaspImmigrant
         builder = proc_cls.get_builder_from_folder(code, str(remote_path), **kwargs)
@@ -568,12 +532,12 @@ class VaspCalculation(VaspCalcBase):
         builder.metadata['options']['resources'] = resources  # pylint: disable=no-member
         settings = kwargs.get('settings', {})
         settings.update({'import_from_path': str(remote_path)})
-        builder.settings = get_data_node('core.dict', dict=settings)
+        builder.settings = orm.Dict(dict=settings)
 
         return proc_cls, builder
 
 
-def ordered_unique_list(in_list):
+def ordered_unique_list(in_list: list) -> list:
     """List unique elements in input list, in order of first occurrence."""
     out_list = []
     for i in in_list:
@@ -582,7 +546,7 @@ def ordered_unique_list(in_list):
     return out_list
 
 
-def ordered_unique_symbols(structure):
+def ordered_unique_symbols(structure: orm.StructureData) -> list[str]:
     """
     Return a list of ordered unique symbols in the structure
     """
