@@ -1,10 +1,12 @@
 """
-The vasprun.xml parser interface.
+The ``vasprun.xml`` parser interface.
 
----------------------------------
 Contains the parsing interfaces to ``parsevasp`` used to parse ``vasprun.xml`` content.
 """
+
 # pylint: disable=abstract-method, too-many-public-methods
+from typing import Any, BinaryIO, TextIO
+
 import numpy as np
 from parsevasp import constants as parsevaspct
 from parsevasp.vasprun import Xml
@@ -38,41 +40,24 @@ class VasprunParser(BaseFileParser):
             'forces',
             'stress',
             'total_energies',
-            'maximum_force',
-            'maximum_stress',
             'band_properties',
             'version',
+            'parameters',
         ],
         'energy_type': ['energy_extrapolated'],
-        'electronic_step_energies':
-        False
+        'electronic_step_energies': False,
     }
 
     PARSABLE_QUANTITIES = {
-        'structure': {
-            'inputs': [],
-            'name': 'structure',
-            'prerequisites': [],
-            'alternatives': ['poscar-structure']
-        },
+        'structure': {'inputs': [], 'name': 'structure', 'prerequisites': [], 'alternatives': ['poscar-structure']},
         'eigenvalues': {
             'inputs': [],
             'name': 'eigenvalues',
             'prerequisites': [],
-            'alternatives': ['eigenval-eigenvalues']
-        },
-        'dos': {
-            'inputs': [],
-            'name': 'dos',
-            'prerequisites': [],
-            'alternatives': ['doscar-dos']
-        },
-        'kpoints': {
-            'inputs': [],
-            'name': 'kpoints',
-            'prerequisites': [],
-            'alternatives': ['kpoints-kpoints']
+            'alternatives': ['eigenval-eigenvalues'],
         },
+        'dos': {'inputs': [], 'name': 'dos', 'prerequisites': [], 'alternatives': ['doscar-dos']},
+        'kpoints': {'inputs': [], 'name': 'kpoints', 'prerequisites': [], 'alternatives': ['kpoints-kpoints']},
         'occupancies': {
             'inputs': [],
             'name': 'occupancies',
@@ -133,16 +118,6 @@ class VasprunParser(BaseFileParser):
             'name': 'fermi_level',
             'prerequisites': [],
         },
-        'maximum_force': {
-            'inputs': [],
-            'name': 'maximum_force',
-            'prerequisites': []
-        },
-        'maximum_stress': {
-            'inputs': [],
-            'name': 'maximum_stress',
-            'prerequisites': []
-        },
         'band_properties': {
             'inputs': [],
             'name': 'band_properties',
@@ -152,7 +127,12 @@ class VasprunParser(BaseFileParser):
             'inputs': [],
             'name': 'version',
             'prerequisites': [],
-        }
+        },
+        'parameters': {
+            'inputs': [],
+            'name': 'parameters',
+            'prerequisites': [],
+        },
     }
 
     # Mapping of the energy names to those returned by parsevasp.vasprunl.Xml
@@ -175,7 +155,7 @@ class VasprunParser(BaseFileParser):
         'energy_no_entropy_electronic': 'energy_no_entropy',
     }
 
-    def _init_from_handler(self, handler):
+    def _init_from_handler(self, handler: BinaryIO | TextIO) -> None:
         """Initialize using a file like handler."""
 
         self.overflow = False
@@ -185,12 +165,13 @@ class VasprunParser(BaseFileParser):
             if exception.code == 509:
                 # Xml might be fine but overflow is detected
                 self.overflow = True
+                self.parser_notifications['vasprun_xml_overflow'] = True
                 self._logger.warning('Parsevasp exited abnormally due to overflow in XML file.')
             else:
                 self._logger.warning('Parsevasp exited abnormally.')
 
     @property
-    def version(self):
+    def version(self) -> str | None:
         """Fetch the VASP version from ``parsevasp`` and return it as a string object."""
 
         # fetch version
@@ -202,7 +183,7 @@ class VasprunParser(BaseFileParser):
         return version
 
     @property
-    def eigenvalues(self):
+    def eigenvalues(self) -> dict[str, Any] | None:
         """Fetch eigenvalues."""
 
         # Fetch eigenvalues
@@ -214,7 +195,7 @@ class VasprunParser(BaseFileParser):
         return eigenvalues
 
     @property
-    def occupancies(self):
+    def occupancies(self) -> dict[str, Any] | None:
         """Fetch occupancies."""
 
         # Fetch occupancies
@@ -227,7 +208,7 @@ class VasprunParser(BaseFileParser):
         return occupancies
 
     @property
-    def kpoints(self):
+    def kpoints(self) -> dict[str, Any] | None:
         """Fetch the kpoints an prepare for consumption by the NodeComposer."""
 
         kpts = self._content_parser.get_kpoints()
@@ -247,7 +228,7 @@ class VasprunParser(BaseFileParser):
         return kpoints_data
 
     @property
-    def structure(self):
+    def structure(self) -> dict[str, Any] | None:
         """
         Fetch a given structure.
 
@@ -264,7 +245,7 @@ class VasprunParser(BaseFileParser):
         return self.last_structure
 
     @property
-    def last_structure(self):
+    def last_structure(self) -> dict[str, Any] | None:
         """
         Fetch the structure.
 
@@ -278,7 +259,7 @@ class VasprunParser(BaseFileParser):
         return _build_structure(last_lattice)
 
     @property
-    def final_structure(self):
+    def final_structure(self) -> dict[str, Any] | None:
         """
         Fetch the structure.
 
@@ -290,7 +271,7 @@ class VasprunParser(BaseFileParser):
         return self.last_structure
 
     @property
-    def last_forces(self):
+    def last_forces(self) -> np.ndarray | None:
         """
         Fetch forces.
 
@@ -302,7 +283,7 @@ class VasprunParser(BaseFileParser):
         return force
 
     @property
-    def final_forces(self):
+    def final_forces(self) -> np.ndarray | None:
         """
         Fetch forces.
 
@@ -313,33 +294,14 @@ class VasprunParser(BaseFileParser):
         return self.last_forces
 
     @property
-    def forces(self):
+    def forces(self) -> np.ndarray | None:
         """
-        Fetch forces.
-
-        This container should contain all relevant forces.
-        Currently, it only contains the final forces, which can be obtain
-        by the id `final_forces`.
-
+        Fetch final forces.
         """
-
-        final_forces = self.final_forces
-        forces = {'final': final_forces}
-
-        return forces
+        return self.final_forces
 
     @property
-    def maximum_force(self):
-        """Fetch the maximum force of at the last ionic run."""
-
-        forces = self.final_forces
-        if forces is None:
-            return None
-        norm = np.linalg.norm(forces, axis=1)
-        return np.amax(np.abs(norm))
-
-    @property
-    def last_stress(self):
+    def last_stress(self) -> np.ndarray | None:
         """
         Fetch stess.
 
@@ -351,7 +313,7 @@ class VasprunParser(BaseFileParser):
         return stress
 
     @property
-    def final_stress(self):
+    def final_stress(self) -> np.ndarray | None:
         """
         Fetch stress.
 
@@ -362,7 +324,7 @@ class VasprunParser(BaseFileParser):
         return self.last_stress
 
     @property
-    def stress(self):
+    def stress(self) -> np.ndarray | None:
         """
         Fetch stress.
 
@@ -372,22 +334,10 @@ class VasprunParser(BaseFileParser):
 
         """
 
-        final_stress = self.final_stress
-        stress = {'final': final_stress}
-        return stress
+        return self.final_stress
 
     @property
-    def maximum_stress(self):
-        """Fetch the maximum stress of at the last ionic run."""
-
-        stress = self.final_stress
-        if stress is None:
-            return None
-        norm = np.linalg.norm(stress, axis=1)
-        return np.amax(np.abs(norm))
-
-    @property
-    def trajectory(self):
+    def trajectory(self) -> dict[str, Any] | None:
         """
         Fetch unitcells, positions, species, forces and stress.
 
@@ -415,9 +365,13 @@ class VasprunParser(BaseFileParser):
         elements = _invert_dict(parsevaspct.elements)
         symbols = np.asarray([elements[item].title() for item in species.tolist()])
 
-        if (unitcell is not None) and (positions is not None) and \
-           (species is not None) and (forces is not None) and \
-           (stress is not None):
+        if (
+            (unitcell is not None)
+            and (positions is not None)
+            and (species is not None)
+            and (forces is not None)
+            and (stress is not None)
+        ):
             trajectory_data = {}
 
             keys = ('cells', 'positions', 'symbols', 'forces', 'stress', 'steps')
@@ -430,7 +384,7 @@ class VasprunParser(BaseFileParser):
         return None
 
     @property
-    def total_energies(self):
+    def total_energies(self) -> dict[str, float] | None:
         """Fetch the total energies after the last ionic run."""
         energies = self.energies
         if energies is None:
@@ -444,7 +398,7 @@ class VasprunParser(BaseFileParser):
         return energies_dict
 
     @property
-    def energies(self):
+    def energies(self) -> dict[str, Any] | None:
         """Fetch the total energies."""
         # Check if we want total energy entries for each electronic step.
         electronic_step_energies = self._settings.get(
@@ -453,7 +407,7 @@ class VasprunParser(BaseFileParser):
 
         return self._energies(nosc=not electronic_step_energies)
 
-    def _energies(self, nosc):
+    def _energies(self, nosc: bool) -> dict[str, Any] | None:
         """
         Fetch the total energies for all energy types, calculations (ionic steps) and electronic steps.
 
@@ -462,8 +416,9 @@ class VasprunParser(BaseFileParser):
         be found in the flattened ndarray where the key `electronic_steps` indicate how many electronic steps
         there is per ionic step. Using the combination, one can rebuild the electronic step energy per ionic step etc.
 
-        Because the VASPrun parser returns both the electronic step energies (at the end of each cycles) and the ionic step
-        energies (_final), we apply a mapping to recovery the naming such that the ionic step energies do not have the suffix,
+        Because the VASPrun parser returns both the electronic step energies (at the end of each cycles) and the ionic
+        step energies (_final), we apply a mapping to recovery the naming such that the ionic step energies do not have
+        the suffix,
         but the electronic step energies do.
         """
 
@@ -492,8 +447,9 @@ class VasprunParser(BaseFileParser):
                 # The energy_extrapolated_final is the entropy term itself in VASP 5
                 # Store the calculated energy_no_entropy under 'energy_extrapolated_final',
                 # which is then recovered as `energy_no_entropy` later
-                energies['energy_extrapolated_final'
-                         ] = energies['energy_free_final'] - energies['energy_extrapolated_final']
+                energies['energy_extrapolated_final'] = (
+                    energies['energy_free_final'] - energies['energy_extrapolated_final']
+                )
         else:
             energies = self._content_parser.get_energies(status='all', etype=etype, nosc=nosc)
 
@@ -516,13 +472,13 @@ class VasprunParser(BaseFileParser):
         return mapped_energies
 
     @property
-    def projectors(self):
+    def projectors(self) -> np.ndarray | None:
         """Fetch the projectors."""
 
         proj = self._content_parser.get_projectors()
         if proj is None:
             return None
-        projectors = {}
+        projectors = None
         prj = []
         try:
             prj.append(proj['total'])  # pylint: disable=unsubscriptable-object
@@ -533,14 +489,14 @@ class VasprunParser(BaseFileParser):
             except KeyError:
                 self._logger.error('Did not detect any projectors. Returning.')
         if len(prj) == 1:
-            projectors['projectors'] = prj[0]
+            projectors = prj[0]
         else:
-            projectors['projectors'] = np.asarray(prj)
+            projectors = np.asarray(prj)
 
         return projectors
 
     @property
-    def dielectrics(self):
+    def dielectrics(self) -> dict[str, Any] | None:
         """Fetch the dielectric function."""
 
         diel = self._content_parser.get_dielectrics()
@@ -566,27 +522,22 @@ class VasprunParser(BaseFileParser):
         return dielectrics
 
     @property
-    def born_charges(self):
+    def born_charges(self) -> np.ndarray | None:
         """Fetch the Born effective charges."""
 
-        brn = self._content_parser.get_born()
-        if brn is None:
-            return None
-        born = {'born_charges': brn}
-        return born
+        return self._content_parser.get_born()
 
     @property
-    def hessian(self):
+    def hessian(self) -> np.ndarray | None:
         """Fetch the Hessian matrix."""
 
         hessian = self._content_parser.get_hessian()
         if hessian is None:
             return None
-        hess = {'hessian': hessian}
-        return hess
+        return hessian
 
     @property
-    def dynmat(self):
+    def dynmat(self) -> dict[str, Any] | None:
         """Fetch the dynamical eigenvectors and eigenvalues."""
 
         dynmat = self._content_parser.get_dynmat()
@@ -598,7 +549,7 @@ class VasprunParser(BaseFileParser):
         return dyn
 
     @property
-    def dos(self):
+    def dos(self) -> dict[str, Any] | None:
         """Fetch the total density of states."""
 
         dos = self._content_parser.get_dos()
@@ -616,8 +567,7 @@ class VasprunParser(BaseFileParser):
         total = dos.get('total')
         if (upspin is not None) and (downspin is not None):
             tdos = np.stack((upspin['total'], downspin['total']))
-            if (upspin['partial'] is not None) and \
-               (downspin['partial'] is not None):
+            if (upspin['partial'] is not None) and (downspin['partial'] is not None):
                 pdos = np.stack((upspin['partial'], downspin['partial']))
         else:
             tdos = total['total']
@@ -629,13 +579,17 @@ class VasprunParser(BaseFileParser):
         return densta
 
     @property
-    def fermi_level(self):
+    def fermi_level(self) -> float | None:
         """Fetch Fermi level."""
 
-        return self._content_parser.get_fermi_level()
+        try:
+            fermi_level = self._content_parser.get_fermi_level()
+        except (ValueError, TypeError):
+            fermi_level = None
+        return fermi_level
 
     @property
-    def run_status(self):
+    def run_status(self) -> dict[str, bool | None]:
         """Fetch run_status information"""
         info = {}
         # First check electronic convergence by comparing executed steps to the
@@ -656,11 +610,10 @@ class VasprunParser(BaseFileParser):
         energies = self._content_parser.get_energies('all', nosc=True)
         if energies is None:
             info['ionic_converged'] = False
+        elif len(energies.get('electronic_steps')) < parameters['nsw'] and not self._content_parser.truncated:
+            info['ionic_converged'] = True
         else:
-            if len(energies.get('electronic_steps')) < parameters['nsw'] and not self._content_parser.truncated:
-                info['ionic_converged'] = True
-            else:
-                info['ionic_converged'] = False
+            info['ionic_converged'] = False
         # Override if nsw is 0 - no ionic steps are performed
         if parameters['nsw'] < 1:
             info['ionic_converged'] = None
@@ -668,7 +621,7 @@ class VasprunParser(BaseFileParser):
         return info
 
     @property
-    def band_properties(self):
+    def band_properties(self) -> dict[str, Any] | None:
         """Fetch key properties of the electronic structure."""
 
         eigenvalues = self.eigenvalues
@@ -686,8 +639,19 @@ class VasprunParser(BaseFileParser):
 
         return get_band_properties(eig, occ)
 
+    @property
+    def parameters(self) -> dict[str, Any] | None:
+        """
+        Fetch the parsed input parameters which will include default values
+        defined in the XML file.
+        """
 
-def _build_structure(lattice):
+        parameters = self._content_parser.get_parameters()
+
+        return parameters
+
+
+def _build_structure(lattice: dict[str, Any]) -> dict[str, Any]:
     """Builds a structure according to AiiDA spec."""
     structure_dict = {}
     structure_dict['unitcell'] = lattice['unitcell']
@@ -705,5 +669,5 @@ def _build_structure(lattice):
     return structure_dict
 
 
-def _invert_dict(dct):
+def _invert_dict(dct: dict[Any, Any]) -> dict[Any, Any]:
     return dct.__class__(map(reversed, dct.items()))
