@@ -1,26 +1,30 @@
 """
 VASP NEB workchain.
 
----------------
 Contains the VaspNEBWorkChain class definition which uses the BaseRestartWorkChain.
 """
-import numpy as np
-#pylint: disable=too-many-branches, too-many-statements
-from packaging import version
 
+from __future__ import annotations
+
+from typing import Any
+
+import numpy as np
 from aiida import __version__ as aiida_version
 from aiida import orm
 from aiida.common.exceptions import InputValidationError, NotExistent
 from aiida.common.extendeddicts import AttributeDict
 from aiida.common.lang import override
-from aiida.engine import while_
+from aiida.engine import ExitCode, ProcessSpec, while_
 from aiida.engine.processes.workchains.restart import BaseRestartWorkChain, ProcessHandlerReport, process_handler
 from aiida.plugins import CalculationFactory
 
+# pylint: disable=too-many-branches, too-many-statements
+from packaging import version
+
 from aiida_vasp.assistant.parameters import ParametersMassage
 from aiida_vasp.calcs.neb import VaspNEBCalculation
+from aiida_vasp.data.potcar import PotcarData
 from aiida_vasp.parsers.content_parsers.potcar import MultiPotcarIo
-from aiida_vasp.utils.aiida_utils import get_data_class, get_data_node
 from aiida_vasp.utils.workchains import compose_exit_code
 
 # Additional tags for VTST calculations - these are not the tags used by standard VASP
@@ -39,15 +43,20 @@ VTST_ADDITIONAL_TAGS = {
     'ftimedec': 'Factor to decrease dt',
     'ftimeinc': 'Factor to increase dt',
     'falpha': 'Parameter that controls velocity damping',
-    'fnmin': 'Minium number of iterations before adjust alpha and dt'
+    'fnmin': 'Minium number of iterations before adjust alpha and dt',
+    'lclimb': 'Use climbing image mode',
+    'ichain': 'Indicates which method to run. NEB (ICHAIN=0) is the default',
+    'ltangentold': 'Flag to turn on the old central difference tangent',
+    'ldneb': 'Flag to turn on modified double nudging',
+    'lnebcell': 'Flag to turn on SS-NEB. Used with ISIF=3 and IOPT=3.',
+    'jacobian': 'Controls weight of lattice to atomic motion. Ω is volume and N is the number of atoms.',
 }
 
 
 class VaspNEBWorkChain(BaseRestartWorkChain):
     """
-    The NEB workchain.
+    A NEB workchain
 
-    -------------------
     Error handling enriched wrapper around VaspNEBCalculation.
 
     Deliberately conserves most of the interface (required inputs) of the VaspNEBCalculation class, but
@@ -56,12 +65,13 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
     In addition, implement restarts of calculation when the calculation is net full converged for error handling.
 
     """
+
     _verbose = False
     _process_class = CalculationFactory('vasp.neb')
-    _norm_disp_threshold = 1.0
+    _norm_disp_threshold = 4.0
 
     @classmethod
-    def define(cls, spec):
+    def define(cls, spec: ProcessSpec) -> None:
         super(VaspNEBWorkChain, cls).define(spec)
         spec.expose_inputs(
             cls._process_class,
@@ -69,59 +79,59 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
         )
         spec.input(
             'kpoints',
-            valid_type=get_data_class('core.array.kpoints'),
+            valid_type=orm.KpointsData,
             required=False,
         )
         spec.input(
             'kpoints_spacing',
-            valid_type=get_data_class('core.float'),
+            valid_type=orm.Float,
             required=False,
         )
         spec.input(
             'potential_family',
-            valid_type=get_data_class('core.str'),
+            valid_type=orm.Str,
             required=True,
         )
         spec.input(
             'potential_mapping',
-            valid_type=get_data_class('core.dict'),
+            valid_type=orm.Dict,
             required=True,
         )
         spec.input(
             'options',
-            valid_type=get_data_class('core.dict'),
+            valid_type=orm.Dict,
             required=True,
         )
         spec.input(
             'max_iterations',
-            valid_type=get_data_class('core.int'),
+            valid_type=orm.Int,
             required=False,
-            default=lambda: get_data_node('core.int', 5),
+            default=lambda: orm.Int(5),
             help="""
             The maximum number of iterations to perform.
             """,
         )
         spec.input(
             'clean_workdir',
-            valid_type=get_data_class('core.bool'),
+            valid_type=orm.Bool,
             required=False,
-            default=lambda: get_data_node('core.bool', False),
+            default=lambda: orm.Bool(False),
             help="""
             If True, clean the work dir upon the completion of a successful calculation.
             """,
         )
         spec.input(
             'verbose',
-            valid_type=get_data_class('core.bool'),
+            valid_type=orm.Bool,
             required=False,
-            default=lambda: get_data_node('core.bool', True),
+            default=lambda: orm.Bool(True),
             help="""
             If True, enable more detailed output during workchain execution.
             """,
         )
         spec.input(
             'dynamics.positions_dof',
-            valid_type=get_data_class('core.list'),
+            valid_type=orm.List,
             required=False,
             help="""
             Site dependent flag for selective dynamics when performing relaxation
@@ -129,31 +139,30 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
         )
         spec.input(
             'ldau_mapping',
-            valid_type=get_data_class('core.dict'),
+            valid_type=orm.Dict,
             required=False,
             help="Mappings, see the doc string of 'get_ldau_keys'",
         )
         spec.input(
             'kpoints_spacing',
-            valid_type=get_data_class('core.float'),
+            valid_type=orm.Float,
             required=False,
             help='Spacing for the kpoints in units A^-1 * 2pi (CASTEP style `kpoints_mp_spacing`)',
         )
         spec.input(
             'kpoints_spacing_vasp',
-            valid_type=get_data_class('core.float'),
+            valid_type=orm.Float,
             required=False,
             help='Spacing for the kpoints in units A^-1 (VASP style)',
         )
         spec.outline(
             cls.setup,
             while_(cls.should_run_process)(
-                #cls.prepare_inputs,
                 cls.run_process,
                 cls.inspect_process,
             ),
             cls.results,
-        )  # yapf: disable
+        )
 
         spec.expose_outputs(cls._process_class)
         spec.exit_code(
@@ -187,8 +196,7 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
             message='Unrecoverable error in launched NEB calculations.',
         )
 
-    def setup(self):
-
+    def setup(self) -> None | ExitCode:
         super().setup()
 
         # Setup the initial inputs
@@ -204,37 +212,27 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
 
         # Sanity checks
         self._check_neb_inputs()
-        #self.report('In SETUP, context metadata {}'.format(self.ctx.inputs))
         return None
 
-    # def prepare_inputs(self):
-    #     """
-    #     Prepare the inputs stored under self.ctx.inputs
-    #     """
-
-    #     # Applied the staged NEB images
-    #     self.ctx.inputs.neb_images = self.ctx.neb_images
-
     @process_handler(priority=500, exit_codes=[VaspNEBCalculation.exit_codes.ERROR_IONIC_NOT_CONVERGED])  # pylint: disable=no-member
-    def handle_unconverged(self, node):
+    def handle_unconverged(self, node: orm.WorkChainNode) -> ProcessHandlerReport | None:
         """
         Handle the problem where the NEB optimization is not converged.
 
         Note that VASP could reach NSW before the actual convergence.
         Hence this check is necessary even for finished runs.
         """
-        if 'neb_misc' not in node.outputs:
-            self.report('Cannot found the `neb_misc` output containing the NEB run data')
+        if 'misc' not in node.outputs:
+            self.report('Cannot found the `misc` output containing the NEB run data')
             return None
-        neb_misc = node.outputs.neb_misc.get_dict()
+        misc_dict = node.outputs.misc.get_dict()
 
-        if not neb_misc.get('neb_data'):
+        neb_data = misc_dict.get('neb_data')
+        if neb_data is None:
             self.report('Cannot found the `neb_data` dictionary containing the NEB run data')
             return None
 
-        neb_data = neb_misc.get('neb_data')
-
-        converged = [tmp['neb_converged'] for tmp in neb_data.values()]
+        converged = [tmp.get('neb_converged', False) for tmp in neb_data.values()]
         if not all(converged):
             self.report('At least one image is not converged in the run. Restart required.')
 
@@ -249,7 +247,7 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
         return None
 
     @process_handler(priority=900, exit_codes=[VaspNEBCalculation.exit_codes.ERROR_DID_NOT_FINISH])  # pylint: disable=no-member
-    def handle_unfinished(self, node):
+    def handle_unfinished(self, node: orm.WorkChainNode) -> ProcessHandlerReport | None:
         """
         Handle the case where the calculations is not fully finished.
         This checks the existing of the run_stats field in the parsed per-image misc output
@@ -261,25 +259,12 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
             if 'misc' not in node.outputs:
                 self.report('Cannot found the `misc` output containing the parsed per-image data')
                 return None
-            for misc in node.outputs['misc'].values():
-                misc = misc.get_dict()
-                if 'run_status' in misc and misc['run_status'].get('finished'):
-                    finished.append(True)
-                else:
-                    finished.append(False)
-        else:
-            if 'misc__image_01' not in node.outputs:
-                self.report('Cannot found the `misc` output containing the parsed per-image data')
-                return None
-            for key in node.outputs:
-                if key.startswith('misc__'):
-                    misc = node.outputs[key].get_dict()
-                    if 'run_stats' in misc and misc['run_status'].get('finished'):
-                        finished.append(True)
-                    else:
-                        finished.append(False)
 
-        if not all(finished):
+            misc_dict = node.outputs.misc.get_dict()
+            if 'run_status' in misc_dict:
+                finished = {key: value.get('finished', False) for key, value in misc_dict['run_status'].items()}
+
+        if not all(finished.values()):
             self.report('At least one image did not reach the end of VASP execution - calculation not finished!')
 
             out = self._attach_output_structure(node)
@@ -292,20 +277,14 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
         self.report(f'Cannot handle unfinished calculation {node}.')
         return None
 
-    def _attach_output_structure(self, node):
+    def _attach_output_structure(self, node: orm.WorkChainNode) -> ProcessHandlerReport | None:
         """
         Attached the output structure of a children node as the inputs for the
         next workchain launch.
         """
         output_images = AttributeDict()  # A dictionary holding the structures with keys like 'image_xx'
-        # For version older than 1.6.3 the rested output namespace is correctly handled
-        # Hence, we need to access directly using the and other than the `__` in the link name.
-        if version.parse(aiida_version) >= version.parse('1.6.3'):
-            output_images = node.outputs['structure']
-        else:
-            for key in node.outputs:
-                if key.startswith('structure__'):
-                    output_images[key.split('__')[1]] = node.outputs[key]
+        output_images = node.outputs['structure']
+
         nout = len(output_images)
         nexists = len(self.inputs.neb_images)
         if nout != nexists:
@@ -315,7 +294,7 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
         self.ctx.inputs.neb_images = output_images
         return None
 
-    def _check_neb_inputs(self):
+    def _check_neb_inputs(self) -> None:
         """
         Perform some simple checks for the NEB inputs
 
@@ -363,7 +342,7 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
 
         last_frame = frames[0]
         # Function for computing the distance using the scaled positions
-        rel_dist = np.vectorize(lambda x: x if x < 0.5 else 1. - x)
+        rel_dist = np.vectorize(lambda x: x if x < 0.5 else 1.0 - x)
         for iframe, frame in enumerate(frames[1:]):
             # Relative displacements
             disp = abs(frame.get_scaled_positions() - last_frame.get_scaled_positions()) % 1.0
@@ -381,11 +360,12 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
                 )
             last_frame = frame
 
-    def _setup_vasp_inputs(self):
+    def _setup_vasp_inputs(self) -> None:
         """
         Setup the inputs for VASP calculation
 
         This method is called once by ``self.setup``
+        TODO: merge with vasp.v2.vasp
         """
 
         # Set the kpoints (kpoints)
@@ -476,10 +456,10 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
             )
             return self.exit_codes.ERROR_NO_POTENTIAL_FAMILY_NAME  # pylint: disable=no-member
         try:
-            self.ctx.inputs.potential = get_data_class('vasp.potcar').get_potcars_from_structure(
+            self.ctx.inputs.potential = PotcarData.get_potcars_from_structure(
                 structure=self.inputs.initial_structure,
                 family_name=self.inputs.potential_family.value,
-                mapping=self.inputs.potential_mapping.get_dict()
+                mapping=self.inputs.potential_mapping.get_dict(),
             )
         except ValueError as err:
             return compose_exit_code(self.exit_codes.ERROR_POTENTIAL_VALUE_ERROR.status, str(err))  # pylint: disable=no-member
@@ -491,7 +471,7 @@ class VaspNEBWorkChain(BaseRestartWorkChain):
         return None
 
     @override
-    def results(self):
+    def results(self) -> ExitCode | None:
         """Attach the outputs specified in the output specification from the last completed process."""
         node = self.ctx.children[self.ctx.iteration - 1]
 
@@ -549,22 +529,26 @@ FELEMS = [
 ]
 
 
-def get_ldau_keys(structure, mapping, utype=2, jmapping=None, felec=False):
+def get_ldau_keys(
+    structure: orm.StructureData,
+    mapping: dict[str, list[float]],
+    utype: int = 2,
+    jmapping: dict[str, float] | None = None,
+    felec: bool = False,
+) -> dict[str, Any]:
     """
-    Setup LDAU mapping. In VASP, the U for each species has to be
-    defined in the order that they appear in POSCAR. This is a helper
-    to make sure the values of U are associated to each specie
+    Setup LDAU mapping.
 
-    Arguments:
-        structure: the structure, either StructureData or ase.Atoms is fine
-        mapping: a dictionary in the format of  {"Mn": [d, 4]...} for U
-        utype: the type of LDA+U, default to 2, which is the one with only one parameter
-        jmapping: a dictionary in the format of  {"Mn": [d, 4]...} but for J
-        felec: Wether we are dealing with f electrons, will increase lmaxmix if we are.
+    In VASP, the U for each species has to be defined in the order that they appear in POSCAR.
+    This is a helper to make sure the values of U are associated to each species.
 
+    :param structure: The structure, either StructureData or ase.Atoms is fine.
+    :param mapping: A dictionary in the format of  ``{"Mn": [d, 4]...}`` for U.
+    :param utype: The type of LDA+U, default to 2, which is the one with only one parameter.
+    :param jmapping: A dictionary in the format of  ``{"Mn": [d, 4]...}`` but for J.
+    :param felec: Whether we are dealing with f electrons; will increase lmaxmix if we are.
 
-    Returns:
-        dict_update: a dictionary to be used to update the raw input parameters for VASP
+    :returns: A dictionary to be used to update the raw input parameters for VASP.
     """
     if isinstance(structure, orm.StructureData):
         species = MultiPotcarIo.potentials_order(structure)
@@ -586,22 +570,22 @@ def get_ldau_keys(structure, mapping, utype=2, jmapping=None, felec=False):
     for specie in species:
         if specie in mapping:
             uvalue = mapping[specie][1]
-            j = jmapping.get(specie, 0.)
+            j = jmapping.get(specie, 0.0)
             ldaul.append(lsymbols[mapping[specie][0]])
             ldauu.append(mapping[specie][1])
 
-            j = jmapping.get(specie, 0.)
+            j = jmapping.get(specie, 0.0)
             ldauj.append(j)
 
             if specie in FELEMS:
                 felec = True
             # Count the number of valid mappings
-            if uvalue != 0. or j != 0.:
+            if uvalue != 0.0 or j != 0.0:
                 count += 1
 
         else:
-            ldauu.append(0.)
-            ldauj.append(0.)
+            ldauu.append(0.0)
+            ldauj.append(0.0)
             ldaul.append(-1)
 
     if count > 0:
@@ -612,7 +596,7 @@ def get_ldau_keys(structure, mapping, utype=2, jmapping=None, felec=False):
             'ldautype': utype,
             'lmaxmix': 6 if felec else 4,
             'ldaul': ldaul,
-            'ldau': True
+            'ldau': True,
         }
     else:
         output = {}
